
uniform sampler2D ColorTexture;
uniform float ITime;
uniform float Scale;
uniform float Typ;

#define TAU 6.28318530718
#define MAX_ITER 15
#define SHOW_TILING


float random (in vec2 st) {
	return fract(sin(dot(st.xy,
		vec2(12.9898, 78.233))) *
		43758.5453123);
}

float noise (in vec2 st) {
	vec2 i = floor(st);
	vec2 f = fract(st);

	// Four corners in 2D of a tile
	float a = random(i);
	float b = random(i + vec2(1.0, 0.0));
	float c = random(i + vec2(0.0, 1.0));
	float d = random(i + vec2(1.0, 1.0));

	vec2 u = f * f * (3.0 - 2.0 * f);

	return mix(a, b, u.x) +
		(c - a) * u.y * (1.0 - u.x) +
		(d - b) * u.x * u.y;
}

#define OCTAVES 1
float fbm (in vec2 st) {
	// Initial values
	float value = 0.0;
	float amplitude = .125;
	float frequency = 0.;
	//
	// Loop of octaves
	for (int i = 0; i < OCTAVES; i++) {
		value += amplitude * noise(st);
		st *= 2.;
		amplitude *= .5;
	}
	return value * 0.5 + 0.5;
}


//Modified from : David Hoskins

void shader(){
	vec2 texcoords;	
	vec4 color;	
	
	
	///////////////////
	
	// SINE
	if(Typ==1) {
		texcoords=(b3d_ClipPosition.st/b3d_ClipPosition.w)*0.5+0.5;
		//texcoords.x += sin(texcoords.y*8+(ITime*4))/Scale;	//25.0;
		texcoords.x += sin(texcoords.y*8+(ITime*3))/Scale;//25.0;
		texcoords.y += sin(texcoords.x*4+(ITime*3))/Scale/2;
		color=texture2D(ColorTexture, texcoords); 

	}
	
	// FLAT
	if(Typ==0) {
		texcoords=(b3d_ClipPosition.st/b3d_ClipPosition.w)*0.5+0.5;
		color=texture2D( ColorTexture,texcoords ).rgba;

	}
	
	if(Typ==2) {
	
		// Normalized pixel coordinates (from 0 to 1)
		vec2 uv = (b3d_ClipPosition.st / b3d_ClipPosition.w) * 0.5 + 0.5; //fragCoord/iResolution.xy;
	    
		float p = 1.1 * fbm(1. * uv + ITime);
		float q = 11. * fbm(uv * 3. * p + ITime * 3);
		float r = fbm(uv * 5. * q);
	    
		uv += (r * p - q / 8.0) + 0.43;//*0.6; //uv-r*p-q/8.0;
	
		// Time varying pixel color
		vec3 col = 0.5 + 0.5*cos(ITime+uv.xyx+vec3(0,2,4));
		color = texture2D(ColorTexture, uv);
	
	}
	
	b3d_FragColor=color;
	
	float time = ITime * .5+23.0;
    // uv should be the 0-1 uv of texture...
	vec2 uv = texcoords; //fragCoord.xy / iResolution.xy;  //aw
    
	float resscale = 2; 
	
#ifdef SHOW_TILING
	vec2 p = mod(uv*TAU*resscale, TAU)-250.0;
#else
    vec2 p = mod(uv*TAU, TAU)-250.0;
#endif
	vec2 i = vec2(p);
	float c = 1.0;
	float inten = .005;

	for (int n = 0; n < MAX_ITER; n++) 
	{
		float t = time * (1.0 - (3.5 / float(n+1)));
		i = p + vec2(cos(t - i.x) + sin(t + i.y), sin(t - i.y) + cos(t + i.x));
		c += 1.0/length(vec2(p.x / (sin(i.x+t)/inten),p.y / (cos(i.y+t)/inten)));
	}
	c /= float(MAX_ITER);
	c = 1.17-pow(c, 1.4);
	vec3 colour = vec3(pow(abs(c), 8.0));
    colour = clamp(colour + vec3(0.0, 0.35, 0.5), 0.0, 1.0);
    

#ifdef SHOW_TILING
	// Flash tile borders...
	vec2 pixel = resscale / b3d_ClipPosition.xy; //iResolution.xy;
	uv *= resscale;

	float f = floor(mod(ITime*.5, resscale)); 	// Flash value.
	vec2 first = step(pixel, uv) * f;		   	// Rule out first screen pixels and flash.
	uv  = step(fract(uv), pixel);				// Add one line of pixels per tile.
	colour = mix(colour, vec3(1.0, 1.0, 0.0), (uv.x + uv.y) * first.x * first.y); // Yellow line

#endif
	if(color.r < 1) {
		b3d_FragColor = vec4(0,0,0,0);	
	}else{
		b3d_FragColor = vec4(colour.r,  colour.r*(sin(ITime)+abs(sin(uv.y+1)))*0.2,  0, 1.0);   //vec4(colour.r,colour.r*sin(ITime)*0.4,0);//, 1.0);
	}
}